<?php
/**
 * Order Status Rules for WooCommerce - Meta Boxes Class
 *
 * @version 1.6.1
 * @since   1.4.0
 *
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Order_Status_Rules_Meta_Boxes' ) ) :

class Alg_WC_Order_Status_Rules_Meta_Boxes {

	/**
	 * Constructor.
	 *
	 * @version 1.4.0
	 * @since   1.4.0
	 */
	function __construct() {
		add_action( 'add_meta_boxes', array( $this, 'add_status_change_meta_box' ) );
	}

	/**
	 * get_core.
	 *
	 * @version 1.6.0
	 * @since   1.6.0
	 */
	function get_core() {
		if ( ! isset( $this->core ) ) {
			$this->core = alg_wc_order_status_rules()->core;
		}
		return $this->core;
	}

	/**
	 * add_status_change_meta_box.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function add_status_change_meta_box() {
		add_meta_box(
			'alg-wc-order-status-change-history',
			__( 'Order Status History', 'order-status-rules-for-woocommerce' ),
			array( $this, 'create_status_change_meta_box' ),
			'shop_order',
			'normal',
			'default'
		);
	}

	/**
	 * create_status_change_meta_box.
	 *
	 * @version 1.6.1
	 * @since   1.0.0
	 *
	 * @todo    [next] [!] show error if last record `to` is not equal to the current order status
	 */
	function create_status_change_meta_box() {
		$order_id       = get_the_ID();
		$status_history = $this->get_core()->get_order_status_change_history( $order_id );
		if ( empty( $status_history ) ) {
			echo '<p><em>' . __( 'No data.', 'order-status-rules-for-woocommerce' ) . '</p></em>';
		} else {
			// Status history
			$date_format    = $this->get_core()->get_date_time_format();
			$status_history = array_reverse( $status_history, true );
			$last_record    = current( $status_history );
			$status         = wc_get_order_statuses();
			echo '<table class="widefat stripped">' .
				'<tr>' .
					'<th>' . __( 'Nr.', 'order-status-rules-for-woocommerce' )  . '</th>' .
					'<th>' . __( 'Time', 'order-status-rules-for-woocommerce' ) . '</th>' .
					'<th>' . __( 'From', 'order-status-rules-for-woocommerce' ) . '</th>' .
					'<th>' . __( 'To', 'order-status-rules-for-woocommerce' )   . '</th>' .
				'</tr>';
			foreach ( $status_history as $index => $record ) {
				$i    = ( $index + 1 );
				$time = date_i18n( $date_format, $record['time'] );
				$from = '<code>' . ( isset( $status[ 'wc-' . $record['from'] ] ) ? $status[ 'wc-' . $record['from'] ] : $record['from'] ) . '</code>';
				$to   = '<code>' . ( isset( $status[ 'wc-' . $record['to'] ] )   ? $status[ 'wc-' . $record['to'] ]   : $record['to'] )   . '</code>';
				echo "<tr><td>{$i}</td><td>{$time}</td><td>{$from}</td><td>{$to}</td></tr>";
			}
			echo '</table>';
			// Scheduled status update
			$this->get_core()->init_options();
			$order        = wc_get_order( $order_id );
			$order_status = $order->get_status();
			$gateway      = $order->get_payment_method();
			foreach ( $this->get_core()->options['from'] as $i => $from ) {
				$args = array(
					'order_status' => $order_status,
					'from'         => $from,
					'last_record'  => $last_record,
					'gateway'      => $gateway,
					'order'        => $order,
				);
				if ( $this->get_core()->do_apply_rule( $i, $args ) ) {
					$unit           = ( isset( $this->get_core()->options['time_trigger_units'][ $i ] ) ? $this->get_core()->options['time_trigger_units'][ $i ] : 'hour' );
					$step           = $this->get_core()->get_trigger_unit_step( $unit );
					$skip           = ( isset( $this->get_core()->options['skip_days'][ $i ] ) ? $this->get_core()->options['skip_days'][ $i ] : false );
					$time_remaining = $this->get_core()->get_time_remaining( $last_record['time'], $this->get_core()->options['time_triggers'][ $i ] * $step, $skip, $step );
					$current_time   = current_time( 'timestamp' );
					$to             = $this->get_core()->options['to'][ $i ];
					$rule           = sprintf( __( 'Rule #%s', 'order-status-rules-for-woocommerce' ), $i ) .
						( ! empty( $this->get_core()->options['titles'][ $i ] ) ? ': ' . $this->get_core()->options['titles'][ $i ] : '' );
					echo '<p><em>' .
						sprintf( __( 'Status scheduled to be updated from %s to %s (%s) on %s (i.e. %s).', 'order-status-rules-for-woocommerce' ),
							'<code>' . ( isset( $status[ $from ] ) ? $status[ $from ] : $from ) . '</code>',
							'<code>' . ( isset( $status[ $to ] )   ? $status[ $to ]   : $to )   . '</code>',
							$rule,
							date_i18n( $date_format, $current_time + $time_remaining ),
							( $time_remaining > 0 ?
								sprintf( __( 'in %s', 'order-status-rules-for-woocommerce' ), human_time_diff( $current_time - $time_remaining, $current_time ) ) :
								__( 'now', 'order-status-rules-for-woocommerce' ) )
						) . ' ' .
						sprintf( __( 'Current time is %s.', 'order-status-rules-for-woocommerce' ),
							date_i18n( $date_format, $current_time ) ) .
					'</p></em>';
					break;
				}
			}
		}
	}

}

endif;

return new Alg_WC_Order_Status_Rules_Meta_Boxes();
