<?php
/**
 * Order Status Rules for WooCommerce - Advanced Section Settings
 *
 * @version 1.8.0
 * @since   1.5.0
 *
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Order_Status_Rules_Settings_Advanced' ) ) :

class Alg_WC_Order_Status_Rules_Settings_Advanced extends Alg_WC_Order_Status_Rules_Settings_Section {

	/**
	 * Constructor.
	 *
	 * @version 1.5.0
	 * @since   1.5.0
	 *
	 * @todo    [next] (dev) split into separate sections: "Advanced", "Tools", "My account", etc.
	 */
	function __construct() {
		$this->id   = 'advanced';
		$this->desc = __( 'Advanced & Tools', 'order-status-rules-for-woocommerce' );
		parent::__construct();
	}

	/**
	 * get_next_scheduled_desc.
	 *
	 * @version 1.4.0
	 * @since   1.2.0
	 *
	 * @todo    [next] (desc) `alg_wc_order_status_rules_no_history`: better title and desc
	 * @todo    [next] (desc) `crons`: desc ("... only when someone visits your site...")
	 * @todo    [maybe] (desc) Advanced Options: better description?
	 */
	function get_next_scheduled_desc() {
		if (
			'yes' === get_option( 'alg_wc_order_status_rules_use_wp_cron', 'yes' ) &&
			false != ( $next_scheduled = wp_next_scheduled( 'alg_wc_order_status_rules_process_rules', array( get_option( 'alg_wc_order_status_rules_wp_cron_schedule', 'hourly' ) ) ) )
		) {
			$date_format    = alg_wc_order_status_rules()->core->get_date_time_format();
			$current_time   = current_time( 'timestamp' );
			$server_time    = time();
			$next_scheduled = $next_scheduled + ( $current_time - $server_time );
			return sprintf( __( 'Next cron event is scheduled on %s (%s).', 'order-status-rules-for-woocommerce' ),
					'<code>' . date_i18n( $date_format, $next_scheduled ) . '</code>',
					( ( $next_scheduled - $current_time ) <= 0 ?
						__( 'i.e. now', 'order-status-rules-for-woocommerce' ) :
						sprintf( __( 'i.e. in %s', 'order-status-rules-for-woocommerce' ), human_time_diff( $next_scheduled, $current_time ) ) )
				) . ' ' .
				sprintf( __( 'Current time is %s.', 'order-status-rules-for-woocommerce' ),
					'<code>' . date_i18n( $date_format, $current_time ) . '</code>' );
		} else {
			return '';
		}
	}

	/**
	 * get_settings.
	 *
	 * @version 1.8.0
	 * @since   1.5.0
	 *
	 * @todo    [maybe] (desc) `alg_wc_order_status_rules_compatibility_doctreat`: better desc, e.g. add link to the theme?
	 */
	function get_settings() {

		$advanced_settings = array(
			array(
				'title'    => __( 'Advanced Options', 'order-status-rules-for-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_order_status_rules_advanced_options',
			),
			array(
				'title'    => __( 'Use WP cron', 'order-status-rules-for-woocommerce' ),
				'desc'     => __( 'Enable', 'order-status-rules-for-woocommerce' ),
				'desc_tip' => sprintf( __( 'This will use %s to process the rules.', 'order-status-rules-for-woocommerce' ),
						'<a href="https://developer.wordpress.org/plugins/cron/" target="_blank">' .
							__( 'WordPress crons', 'order-status-rules-for-woocommerce' ) . '</a>' ) . '<br>' .
					sprintf( __( 'If you are going to disable this option, make sure you have enabled %s option and have set up "real" (i.e. server) cron job instead.', 'order-status-rules-for-woocommerce' ),
						'<span style="text-decoration:underline;">' . __( 'Allow rules processing via URL', 'order-status-rules-for-woocommerce' ) . '</span>' ) . ' ' .
					sprintf( __( 'Otherwise your only option is to %s tool manually.', 'order-status-rules-for-woocommerce' ),
						'<span style="text-decoration:underline;">' . __( 'Run all rules now', 'order-status-rules-for-woocommerce' ) . '</span>' ),
				'id'       => 'alg_wc_order_status_rules_use_wp_cron',
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'desc'     => __( 'WP cron interval', 'order-status-rules-for-woocommerce' ),
				'id'       => 'alg_wc_order_status_rules_wp_cron_schedule',
				'default'  => 'hourly',
				'type'     => 'select',
				'class'    => 'chosen_select',
				'options'  => alg_wc_order_status_rules()->core->crons->get_schedules(),
			),
			array(
				'title'    => __( 'Allow rules processing via URL', 'order-status-rules-for-woocommerce' ),
				'desc'     => __( 'Enable', 'order-status-rules-for-woocommerce' ),
				'desc_tip' => sprintf( __( 'This will allow to initiate all rules processing via URL: %s.', 'order-status-rules-for-woocommerce' ),
					'<code>' . add_query_arg( 'alg_wc_order_status_rules_process_rules', '', get_site_url() ) . '</code>' ) . '<br>' .
					__( 'For example, this could be useful if you are going to disable WP crons and use "real" (i.e. server) cron jobs instead.', 'order-status-rules-for-woocommerce' ),
				'id'       => 'alg_wc_order_status_rules_allow_url',
				'default'  => 'no',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'On empty order status change history', 'order-status-rules-for-woocommerce' ),
				'desc_tip' => __( 'Plugin must be enabled at the time order status change occurs, so there is no order status change history on initial plugin install. This can be solved by using order creation (or modification) date instead (i.e. instead of real status change date). This is ignored for orders with available real status change history.', 'order-status-rules-for-woocommerce' ),
				'id'       => 'alg_wc_order_status_rules_no_history',
				'default'  => 'use_date_modified',
				'type'     => 'select',
				'class'    => 'chosen_select',
				'options'  => array(
					'do_nothing'        => __( 'Do nothing', 'order-status-rules-for-woocommerce' ),
					'use_date_created'  => __( 'Use order date created', 'order-status-rules-for-woocommerce' ),
					'use_date_modified' => __( 'Use order date modified', 'order-status-rules-for-woocommerce' ),
				),
			),
			array(
				'title'    => __( 'Debug', 'order-status-rules-for-woocommerce' ),
				'desc'     => __( 'Enable', 'order-status-rules-for-woocommerce' ),
				'desc_tip' => sprintf( __( 'Will add a log to %s.', 'order-status-rules-for-woocommerce' ),
					'<a target="_blank" href="' . admin_url( 'admin.php?page=wc-status&tab=logs' ) . '">' .
						__( 'WooCommerce > Status > Logs', 'order-status-rules-for-woocommerce' ) . '</a>' ),
				'id'       => 'alg_wc_order_status_rules_debug',
				'default'  => 'no',
				'type'     => 'checkbox',
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_order_status_rules_advanced_options',
			),
		);

		$compatibility_settings = array(
			array(
				'title'    => __( 'Compatibility Options', 'order-status-rules-for-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_order_status_rules_compatibility_options',
			),
			array(
				'title'    => __( '"Doctreat" theme', 'order-status-rules-for-woocommerce' ),
				'desc'     => __( 'Enable', 'order-status-rules-for-woocommerce' ),
				'desc_tip' => __( 'For the "Doctreat - Doctors Directory WordPress Theme" by AmentoTech.', 'order-status-rules-for-woocommerce' ),
				'id'       => 'alg_wc_order_status_rules_compatibility_doctreat',
				'default'  => 'no',
				'type'     => 'checkbox',
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_order_status_rules_compatibility_options',
			),
		);

		$my_account_settings = array(
			array(
				'title'    => __( '"My Account" Options', 'order-status-rules-for-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_order_status_rules_my_account_options',
			),
			array(
				'title'    => __( 'My Account > Orders', 'order-status-rules-for-woocommerce' ),
				'desc'     => '<strong>' . __( 'Enable', 'order-status-rules-for-woocommerce' ) . '</strong>',
				'desc_tip' => __( 'Adds order status history to "My Account > Orders".', 'order-status-rules-for-woocommerce' ),
				'id'       => 'alg_wc_osr_my_account_orders_status_history_enabled',
				'default'  => 'no',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'Position', 'order-status-rules-for-woocommerce' ),
				'id'       => 'alg_wc_osr_my_account_orders_status_history_position',
				'default'  => 'order-status',
				'type'     => 'select',
				'class'    => 'chosen_select',
				'options'  => array(
					'order-status'       => __( '"Status" column', 'order-status-rules-for-woocommerce' ),
					'alg-wc-osr-history' => __( 'New column', 'order-status-rules-for-woocommerce' ),
				),
			),
			array(
				'title'    => __( 'Column title', 'order-status-rules-for-woocommerce' ),
				'desc'     => __( 'Ignored unless "New column" is selected for the "Position".', 'order-status-rules-for-woocommerce' ),
				'id'       => 'alg_wc_osr_my_account_orders_status_history_column_title',
				'default'  => __( 'History', 'order-status-rules-for-woocommerce' ),
				'type'     => 'text',
			),
			array(
				'title'    => __( 'Templates', 'order-status-rules-for-woocommerce' ),
				'desc'     => __( 'Before', 'order-status-rules-for-woocommerce' ) . '<br>' .
					sprintf( __( 'Available placeholders: %s', 'order-status-rules-for-woocommerce' ), '<code>' . implode( '</code>, <code>', array(
							'%current_status%',
						) ) . '</code>' ),
				'id'       => 'alg_wc_osr_my_account_orders_status_history_templates[before]',
				'default'  => '%current_status%',
				'type'     => 'textarea',
			),
			array(
				'desc'     => __( 'Each record', 'order-status-rules-for-woocommerce' ) . '<br>' .
					sprintf( __( 'Available placeholders: %s', 'order-status-rules-for-woocommerce' ), '<code>' . implode( '</code>, <code>', array(
							'%record_nr%',
							'%record_date%',
							'%record_time%',
							'%status_from%',
							'%status_to%',
						) ) . '</code>' ),
				'desc_tip' => sprintf( __( '%s is a HTML code for the right arrow symbol.', 'order-status-rules-for-woocommerce' ), htmlentities( '&amp;rarr;' ) ),
				'id'       => 'alg_wc_osr_my_account_orders_status_history_templates[each_record]',
				'default'  => '<br>%status_from% &rarr; %status_to%',
				'type'     => 'textarea',
			),
			array(
				'desc'     => __( 'After', 'order-status-rules-for-woocommerce' ) . '<br>' .
					sprintf( __( 'Available placeholders: %s', 'order-status-rules-for-woocommerce' ), '<code>' . implode( '</code>, <code>', array(
							'%current_status%',
						) ) . '</code>' ),
				'id'       => 'alg_wc_osr_my_account_orders_status_history_templates[after]',
				'default'  => '',
				'type'     => 'textarea',
			),
			array(
				'title'    => __( 'Reverse status history', 'order-status-rules-for-woocommerce' ),
				'desc'     => __( 'Enable', 'order-status-rules-for-woocommerce' ),
				'id'       => 'alg_wc_osr_my_account_orders_status_history_reverse',
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_order_status_rules_my_account_options',
			),
		);

		$tools = array(
			array(
				'title'    => __( 'Tools', 'order-status-rules-for-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_order_status_rules_tools_options',
				'desc'     => $this->get_next_scheduled_desc(),
			),
			array(
				'title'    => __( 'Run all rules now', 'order-status-rules-for-woocommerce' ),
				'desc'     => '<strong>' . __( 'Run', 'order-status-rules-for-woocommerce' ) . '</strong>',
				'desc_tip' => __( 'Check the box and save changes to run all rules now.', 'order-status-rules-for-woocommerce' ),
				'id'       => 'alg_wc_order_status_rules_run_now',
				'default'  => 'no',
				'type'     => 'checkbox',
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_order_status_rules_tools_options',
			),
		);

		return array_merge( $advanced_settings, $compatibility_settings, $tools, $my_account_settings );
	}

}

endif;

return new Alg_WC_Order_Status_Rules_Settings_Advanced();
