<?php
/**
 * Order Status Rules for WooCommerce - General Section Settings
 *
 * @version 1.8.0
 * @since   1.0.0
 *
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Order_Status_Rules_Settings_General' ) ) :

class Alg_WC_Order_Status_Rules_Settings_General extends Alg_WC_Order_Status_Rules_Settings_Section {

	/**
	 * Constructor.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function __construct() {
		$this->id   = '';
		$this->desc = __( 'General', 'order-status-rules-for-woocommerce' );
		parent::__construct();
	}

	/**
	 * get_settings.
	 *
	 * @version 1.8.0
	 * @since   1.0.0
	 *
	 * @todo    [next] (dev) `alg_wc_order_status_rules()->core->init_options()`?
	 * @todo    [next] (dev) `alg_wc_order_status_rules_products`: ajax (and maybe for `alg_wc_order_status_rules_product_cats`, `alg_wc_order_status_rules_product_tags` as well)
	 * @todo    [next] (desc) `alg_wc_order_status_rules_skip_days`: better desc
	 * @todo    [next] (desc) separate settings section for each rule?
	 * @todo    [maybe] (dev) `alg_wc_order_status_rules_gateways`: add "Other" and/or "N/A" options?
	 * @todo    [maybe] (desc) `alg_wc_order_status_rules_gateways`: better desc?
	 * @todo    [maybe] (feature) `alg_wc_order_status_rules_gateways`, `alg_wc_order_status_rules_products`, `alg_wc_order_status_rules_product_cats`, `alg_wc_order_status_rules_product_tags`: add "exclude" options?
	 */
	function get_settings() {

		$plugin_settings = array(
			array(
				'title'    => __( 'Order Status Rules Options', 'order-status-rules-for-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_order_status_rules_plugin_options',
			),
			array(
				'title'    => __( 'Order Status Rules', 'order-status-rules-for-woocommerce' ),
				'desc'     => '<strong>' . __( 'Enable plugin', 'order-status-rules-for-woocommerce' ) . '</strong>',
				'desc_tip' => __( 'Manage WooCommerce order statuses. Beautifully.', 'order-status-rules-for-woocommerce' ),
				'id'       => 'alg_wc_order_status_rules_plugin_enabled',
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'Total rules', 'order-status-rules-for-woocommerce' ),
				'desc_tip' => __( 'New settings fields will be displayed on the current page if you change this option and "Save changes".', 'order-status-rules-for-woocommerce' ),
				'desc'     => apply_filters( 'alg_wc_order_status_rules_settings',
					sprintf( '<p>' . 'You will need %s plugin to add more than one rule.' . '</p>',
						'<a target="_blank" href="https://wpfactory.com/item/order-status-rules-for-woocommerce/">Order Status Rules for WooCommerce Pro</a>' ) ),
				'id'       => 'alg_wc_order_status_rules_total',
				'default'  => 1,
				'type'     => 'number',
				'custom_attributes' => apply_filters( 'alg_wc_order_status_rules_settings', array( 'readonly' => 'readonly' ), 'rules_total_atts' ),
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_order_status_rules_plugin_options',
			),
		);

		$titles         = get_option( 'alg_wc_order_status_rules_title', array() );
		$_gateways      = WC()->payment_gateways()->payment_gateways;
		$_gateways      = array_combine( wp_list_pluck( $_gateways, 'id' ), wp_list_pluck( $_gateways, 'method_title' ) );
		$products       = get_option( 'alg_wc_order_status_rules_products', array() );
		$_products      = wc_get_products( array( 'limit' => -1, 'return' => 'ids', 'type' => array_merge( array_keys( wc_get_product_types() ), array( 'variation' ) ) ) );
		$_products      = array_combine( $_products, array_map( array( $this, 'get_product_title' ), $_products ) );
		$product_cats   = get_option( 'alg_wc_order_status_rules_product_cats', array() );
		$_product_cats  = get_terms( array( 'taxonomy' => 'product_cat', 'hide_empty' => false ) );
		$_product_cats  = ( ! empty( $_product_cats ) && ! is_wp_error( $_product_cats ) ? array_combine( wp_list_pluck( $_product_cats, 'term_id' ), wp_list_pluck( $_product_cats, 'name' ) ) : array() );
		$product_tags   = get_option( 'alg_wc_order_status_rules_product_tags', array() );
		$_product_tags  = get_terms( array( 'taxonomy' => 'product_tag', 'hide_empty' => false ) );
		$_product_tags  = ( ! empty( $_product_tags ) && ! is_wp_error( $_product_tags ) ? array_combine( wp_list_pluck( $_product_tags, 'term_id' ), wp_list_pluck( $_product_tags, 'name' ) ) : array() );
		$rules_settings = array();
		for ( $i = 1; $i <= apply_filters( 'alg_wc_order_status_rules_rules_total', 1 ); $i++ ) {
			$rules_settings = array_merge( $rules_settings, array(
				array(
					'title'    => sprintf( __( 'Rule #%s', 'order-status-rules-for-woocommerce' ), $i ) . ( ! empty( $titles[ $i ] ) ? ': ' . $titles[ $i ] : '' ),
					'type'     => 'title',
					'id'       => "alg_wc_order_status_rules_options[{$i}]",
				),
				array(
					'title'    => __( 'Enable/Disable', 'order-status-rules-for-woocommerce' ),
					'desc'     => '<strong>' . sprintf( __( 'Enable rule #%s', 'order-status-rules-for-woocommerce' ), $i ) . '</strong>',
					'id'       => "alg_wc_order_status_rules_enabled[{$i}]",
					'default'  => 'no',
					'type'     => 'checkbox',
				),
				array(
					'title'    => __( 'Status from', 'order-status-rules-for-woocommerce' ),
					'id'       => "alg_wc_order_status_rules_from[{$i}]",
					'default'  => 'wc-pending',
					'type'     => 'select',
					'class'    => 'wc-enhanced-select',
					'options'  => wc_get_order_statuses(),
				),
				array(
					'title'    => __( 'Status to', 'order-status-rules-for-woocommerce' ),
					'id'       => "alg_wc_order_status_rules_to[{$i}]",
					'default'  => 'wc-cancelled',
					'type'     => 'select',
					'class'    => 'wc-enhanced-select',
					'options'  => wc_get_order_statuses(),
				),
				array(
					'title'    => __( 'Time trigger', 'order-status-rules-for-woocommerce' ),
					'id'       => "alg_wc_order_status_rules_time_trigger[{$i}]",
					'default'  => 1,
					'type'     => 'number',
					'custom_attributes' => array( 'min' => 0 ),
				),
				array(
					'id'       => "alg_wc_order_status_rules_time_trigger_unit[{$i}]",
					'default'  => 'hour',
					'type'     => 'select',
					'class'    => 'chosen_select',
					'options'  => array(
						'second' => __( 'second(s)', 'order-status-rules-for-woocommerce' ),
						'minute' => __( 'minute(s)', 'order-status-rules-for-woocommerce' ),
						'hour'   => __( 'hour(s)', 'order-status-rules-for-woocommerce' ),
						'day'    => __( 'day(s)', 'order-status-rules-for-woocommerce' ),
						'week'   => __( 'week(s)', 'order-status-rules-for-woocommerce' ),
					),
				),
				array(
					'title'    => __( 'Skip days', 'order-status-rules-for-woocommerce' ),
					'desc_tip' => __( 'Ignored if empty, or if all seven days are selected.', 'order-status-rules-for-woocommerce' ),
					'id'       => "alg_wc_order_status_rules_skip_days[{$i}]",
					'default'  => array(),
					'type'     => 'multiselect',
					'class'    => 'chosen_select',
					'options'  => array(
						1 => __( 'Monday', 'order-status-rules-for-woocommerce' ),
						2 => __( 'Tuesday', 'order-status-rules-for-woocommerce' ),
						3 => __( 'Wednesday', 'order-status-rules-for-woocommerce' ),
						4 => __( 'Thursday', 'order-status-rules-for-woocommerce' ),
						5 => __( 'Friday', 'order-status-rules-for-woocommerce' ),
						6 => __( 'Saturday', 'order-status-rules-for-woocommerce' ),
						7 => __( 'Sunday', 'order-status-rules-for-woocommerce' ),
					),
				),
				array(
					'title'    => __( 'Payment gateways', 'order-status-rules-for-woocommerce' ),
					'desc_tip' => __( 'Required payment gateways.', 'order-status-rules-for-woocommerce' ) . ' ' .
						__( 'If you want the rule to be applied only for orders with selected payment gateways, you can set them here.', 'order-status-rules-for-woocommerce' ) . ' ' .
						__( 'Ignored if empty.', 'order-status-rules-for-woocommerce' ),
					'id'       => "alg_wc_order_status_rules_gateways[{$i}]",
					'default'  => array(),
					'type'     => 'multiselect',
					'class'    => 'chosen_select',
					'options'  => $_gateways,
				),
				array(
					'title'    => __( 'Products', 'order-status-rules-for-woocommerce' ),
					'desc_tip' => __( 'Required products.', 'order-status-rules-for-woocommerce' ) . ' ' .
						__( 'If you want the rule to be applied only for orders with selected products, you can set them here.', 'order-status-rules-for-woocommerce' ) . ' ' .
						__( 'Ignored if empty.', 'order-status-rules-for-woocommerce' ),
					'id'       => "alg_wc_order_status_rules_products[{$i}]",
					'default'  => array(),
					'type'     => 'multiselect',
					'class'    => 'chosen_select',
					'options'  => array_replace(
						( ! empty( $products[ $i ] ) ? array_combine( $products[ $i ], array_map( array( $this, 'get_missing_product_title' ), $products[ $i ] ) ) : array() ),
						$_products ),
				),
				array(
					'title'    => __( 'Product categories', 'order-status-rules-for-woocommerce' ),
					'desc_tip' => __( 'Required product categories.', 'order-status-rules-for-woocommerce' ) . ' ' .
						__( 'If you want the rule to be applied only for orders with selected product categories, you can set them here.', 'order-status-rules-for-woocommerce' ) . ' ' .
						__( 'Ignored if empty.', 'order-status-rules-for-woocommerce' ),
					'id'       => "alg_wc_order_status_rules_product_cats[{$i}]",
					'default'  => array(),
					'type'     => 'multiselect',
					'class'    => 'chosen_select',
					'options'  => array_replace(
						( ! empty( $product_cats[ $i ] ) ? array_combine( $product_cats[ $i ], array_map( array( $this, 'get_missing_product_cat_title' ),$product_cats[ $i ] ) ) : array() ),
						$_product_cats ),
				),
				array(
					'title'    => __( 'Product tags', 'order-status-rules-for-woocommerce' ),
					'desc_tip' => __( 'Required product tags.', 'order-status-rules-for-woocommerce' ) . ' ' .
						__( 'If you want the rule to be applied only for orders with selected product tags, you can set them here.', 'order-status-rules-for-woocommerce' ) . ' ' .
						__( 'Ignored if empty.', 'order-status-rules-for-woocommerce' ),
					'id'       => "alg_wc_order_status_rules_product_tags[{$i}]",
					'default'  => array(),
					'type'     => 'multiselect',
					'class'    => 'chosen_select',
					'options'  => array_replace(
						( ! empty( $product_tags[ $i ] ) ? array_combine( $product_tags[ $i ], array_map( array( $this, 'get_missing_product_tag_title' ),$product_tags[ $i ] ) ) : array() ),
						$_product_tags ),
				),
				array(
					'title'    => __( 'Admin title', 'order-status-rules-for-woocommerce' ) . ' (' . __( 'optional', 'order-status-rules-for-woocommerce' ) . ')',
					'id'       => "alg_wc_order_status_rules_title[{$i}]",
					'default'  => '',
					'type'     => 'text',
				),
				array(
					'type'     => 'sectionend',
					'id'       => "alg_wc_order_status_rules_options[{$i}]",
				),
			) );
		}

		return array_merge( $plugin_settings, $rules_settings );
	}

}

endif;

return new Alg_WC_Order_Status_Rules_Settings_General();
